#include "Cube.h"

using namespace Aplimat::Mesh;

CubeMesh::CubeMesh()
{
	this->Position = Vector3(0.0f, 0.0f, 0.0f);
	this->ColliderOffset = Vector3(0.5f, 0.5f, 0.5f);
}

CubeMesh::CubeMesh(float PosX, float PosY, float PosZ)
{
	this->Position = Vector3(PosX, PosY, PosZ);
	this->ColliderOffset = Vector3(0.5f, 0.5f, 0.5f);
}

CubeMesh::CubeMesh(float PosX, float PosY, float PosZ,
	float ColOffX, float ColOffY, float ColOffZ)
{
	this->Position = Vector3(PosX, PosY, PosZ);
	this->ColliderOffset =
		Vector3(ColOffX, ColOffY, ColOffZ);
}

void CubeMesh::Draw()
{
	glBegin(GL_TRIANGLE_STRIP);
		//Front face
		glVertex3f(this->Position.X - 0.5f, this->Position.Y + 0.5f, this->Position.Z + 0.5f);
		glVertex3f(this->Position.X - 0.5f, this->Position.Y - 0.5f, this->Position.Z + 0.5f);
		glVertex3f(this->Position.X + 0.5f, this->Position.Y + 0.5f, this->Position.Z + 0.5f);
		glVertex3f(this->Position.X + 0.5f, this->Position.Y - 0.5f, this->Position.Z + 0.5f);
		//Right face
		glVertex3f(this->Position.X + 0.5f, this->Position.Y + 0.5f, this->Position.Z - 0.5f);
		glVertex3f(this->Position.X + 0.5f, this->Position.Y - 0.5f, this->Position.Z - 0.5f);
		//Back face
		glVertex3f(this->Position.X - 0.5f, this->Position.Y + 0.5f, this->Position.Z - 0.5f);
		glVertex3f(this->Position.X - 0.5f, this->Position.Y - 0.5f, this->Position.Z - 0.5f);
		//Left face
		glVertex3f(this->Position.X - 0.5f, this->Position.Y + 0.5f, this->Position.Z + 0.5f);
		glVertex3f(this->Position.X - 0.5f, this->Position.Y - 0.5f, this->Position.Z + 0.5f);
	glEnd();
	glBegin(GL_TRIANGLE_STRIP);
		//Top face
		glVertex3f(this->Position.X - 0.5f, this->Position.Y + 0.5f, this->Position.Z + 0.5f);
		glVertex3f(this->Position.X + 0.5f, this->Position.Y + 0.5f, this->Position.Z + 0.5f);
		glVertex3f(this->Position.X - 0.5f, this->Position.Y + 0.5f, this->Position.Z - 0.5f);
		glVertex3f(this->Position.X + 0.5f, this->Position.Y + 0.5f, this->Position.Z - 0.5f);		
	glEnd();
	glBegin(GL_TRIANGLE_STRIP);
		//Bottom face
		glVertex3f(this->Position.X - 0.5f, this->Position.Y - 0.5f, this->Position.Z + 0.5f);
		glVertex3f(this->Position.X + 0.5f, this->Position.Y - 0.5f, this->Position.Z + 0.5f);
		glVertex3f(this->Position.X - 0.5f, this->Position.Y - 0.5f, this->Position.Z - 0.5f);
		glVertex3f(this->Position.X + 0.5f, this->Position.Y - 0.5f, this->Position.Z - 0.5f);		
	glEnd();
}

bool CubeMesh::HasCollidedWith(const CubeMesh &object)
{
	//if (A1 > B2 || A2 < B1) NO COLLISION
	bool xHasNotCollided =
		this->Position.X - this->ColliderOffset.X >		//A1
		object.Position.X + object.ColliderOffset.X	||	//B2
		this->Position.X + this->ColliderOffset.X <		//A2
		object.Position.X - object.ColliderOffset.X;	//B1
	bool yHasNotCollided =
		this->Position.Y - this->ColliderOffset.Y >		//A1
		object.Position.Y + object.ColliderOffset.Y	||	//B2
		this->Position.Y + this->ColliderOffset.Y <		//A2
		object.Position.Y - object.ColliderOffset.Y;	//B1
	bool zHasNotCollided =
		this->Position.Z - this->ColliderOffset.Z >		//A1
		object.Position.Z + object.ColliderOffset.Z	||	//B2
		this->Position.Z + this->ColliderOffset.Z <		//A2
		object.Position.Z - object.ColliderOffset.Z;	//B1

	//int x, y, z;
	//if (xHasNotCollided) x=0; else x=1;
	//if (yHasNotCollided) y=0; else y=1;
	//if (zHasNotCollided) z=0; else z=1;
	//printf("%d, %d, %d\n", x, y, z);

	//if (xHasNotCollided == false &&
	//	yHasNotCollided == false &&
	//	ZHasNotCollided == false)
	if (xHasNotCollided ||
		yHasNotCollided ||
		zHasNotCollided)
		return false;
	else
		return true;

}