#include "Vector3.h"

using namespace Aplimat::VectorMath;

Vector3::Vector3()
{
	this->X = 0.0f;
	this->Y = 0.0f;
	this->Z = 0.0f;
}

Vector3::Vector3(float x, float y, float z)
{
	this->X = x;
	this->Y = y;
	this->Z = z;
}

Vector3::~Vector3() { }

Vector3 &Vector3::operator=(const Vector3 &rhs)
{
	this->X = rhs.X;
	this->Y = rhs.Y;
	this->Z = rhs.Z;

	return *this;
}

Vector3 &Vector3::operator+=(const Vector3 &rhs)
{
	this->X += rhs.X;
	this->Y += rhs.Y;
	this->Z += rhs.Z;

	return *this;
}

Vector3 Vector3::operator+(const Vector3 &op)
{
	Vector3 rhs;
	rhs.X = this->X + op.X;
	rhs.Y = this->Y + op.Y;
	rhs.Z = this->Z + op.Z;

	return rhs;
}

Vector3 &Vector3::operator-=(const Vector3 &rhs)
{
	this->X -= rhs.X;
	this->Y -= rhs.Y;
	this->Z -= rhs.Z;

	return *this;
}

Vector3 Vector3::operator-(const Vector3 &op)
{
	Vector3 rhs;
	rhs.X = this->X - op.X;
	rhs.Y = this->Y - op.Y;
	rhs.Z = this->Z - op.Z;

	return rhs;
}

Vector3 &Vector3::operator*=(const float scalar)
{
	this->X *= scalar;
	this->Y *= scalar;
	this->Z *= scalar;

	return *this;
}

Vector3 Vector3::operator*(const float scalar)
{
	Vector3 rhs;
	rhs.X = this->X * scalar;
	rhs.Y = this->Y * scalar;
	rhs.Z = this->Z * scalar;

	return rhs;
}

float Vector3::GetLength()
{
	float d = sqrt(this->X * this->X +
				this->Y * this->Y +
				this->Z * this->Z);

	return d;
}

void Vector3::Normalize()
{
	float length = this->GetLength();
	this->X /= length;
	this->Y /= length;
	this->Z /= length;

	return;
}

Vector3 Vector3::Normalized()
{
	Vector3 normalized(0.0f, 0.0f, 0.0f);
	normalized.X = this->X;
	normalized.Y = this->Y;
	normalized.Z = this->Z;
	float length = this->GetLength();
	normalized.X /= length;
	normalized.Y /= length;
	normalized.Z /= length;

	return normalized;
}

void Vector3::PrintLine()
{
	printf("X:%f, Y:%f, Z:%f\n", this->X, this->Y, this->Z);
	return;
}